@@ -1,4 +1,5 @@
 """Plugin common functions."""
+from abc import ABCMeta
 import logging
 import re
 import shutil
@@ -7,16 +8,16 @@ from typing import List
 
 from josepy import util as jose_util
 import pkg_resources
-import zope.interface
 
 from certbot import achallenges
 from certbot import crypto_util
 from certbot import errors
-from certbot import interfaces
 from certbot import reverter
 from certbot._internal import constants
 from certbot.compat import filesystem
 from certbot.compat import os
+from certbot.interfaces import Installer as AbstractInstaller
+from certbot.interfaces import Plugin as AbstractPlugin
 from certbot.plugins.storage import PluginStorage
 
 logger = logging.getLogger(__name__)
@@ -39,13 +40,11 @@ hostname_regex = re.compile(
     r"^(([a-z0-9]|[a-z0-9][a-z0-9\-]*[a-z0-9])\.)*[a-z]+$", re.IGNORECASE)
 
 
-@zope.interface.implementer(interfaces.IPlugin)
-class Plugin:
+class Plugin(AbstractPlugin, metaclass=ABCMeta):
     """Generic plugin."""
-    # provider is not inherited, subclasses must define it on their own
-    # @zope.interface.provider(interfaces.IPluginFactory)
 
     def __init__(self, config, name):
+        super().__init__(config, name)
         self.config = config
         self.name = name
 
@@ -63,7 +62,7 @@ class Plugin:
     def inject_parser_options(cls, parser, name):
         """Inject parser options.
 
-        See `~.IPlugin.inject_parser_options` for docs.
+        See `~.certbot.interfaces.Plugin.inject_parser_options` for docs.
 
         """
         # dummy function, doesn't check if dest.startswith(self.dest_namespace)
@@ -97,8 +96,7 @@ class Plugin:
         """Find a configuration value for variable ``var``."""
         return getattr(self.config, self.dest(var))
 
-    def auth_hint(self, failed_achalls):
-        # type: (List[achallenges.AnnotatedChallenge]) -> str
+    def auth_hint(self, failed_achalls: List[achallenges.AnnotatedChallenge]) -> str:
         """Human-readable string to help the user troubleshoot the authenticator.
 
         Shown to the user if one or more of the attempted challenges were not a success.
@@ -106,7 +104,7 @@ class Plugin:
         Should describe, in simple language, what the authenticator tried to do, what went
         wrong and what the user should try as their "next steps".
 
-        TODO: auth_hint belongs in IAuthenticator but can't be added until the next major
+        TODO: auth_hint belongs in Authenticator but can't be added until the next major
         version of Certbot. For now, it lives in .Plugin and auth_handler will only call it
         on authenticators that subclass .Plugin. For now, inherit from `.Plugin` to implement
         and/or override the method.
@@ -119,13 +117,13 @@ class Plugin:
         # This is a fallback hint. Authenticators should implement their own auth_hint that
         # addresses the specific mechanics of that authenticator.
         challs = " and ".join(sorted({achall.typ for achall in failed_achalls}))
-        return ("The Certificate Authority couldn't exterally verify that the {name} plugin "
+        return ("The Certificate Authority couldn't externally verify that the {name} plugin "
                 "completed the required {challs} challenges. Ensure the plugin is configured "
                 "correctly and that the changes it makes are accessible from the internet."
                 .format(name=self.name, challs=challs))
 
 
-class Installer(Plugin):
+class Installer(AbstractInstaller, Plugin, metaclass=ABCMeta):
     """An installer base class with reverter and ssl_dhparam methods defined.
 
     Installer plugins do not have to inherit from this class.
